@@ -1,4 +1,4 @@
-/* Copyright 2011--2018 The Tor Project
+/* Copyright 2011--2020 The Tor Project
  * See LICENSE for licensing information */
 
 package org.torproject.descriptor;
@@ -103,7 +103,7 @@ public final class DescriptorSourceFactory {
    *
    * @since 1.0.0
    */
-  public static final DescriptorParser createDescriptorParser() {
+  public static DescriptorParser createDescriptorParser() {
     return (DescriptorParser) retrieve(PARSER_PROPERTY);
   }
 
@@ -113,7 +113,7 @@ public final class DescriptorSourceFactory {
    *
    * @since 1.0.0
    */
-  public static final DescriptorReader createDescriptorReader() {
+  public static DescriptorReader createDescriptorReader() {
     return (DescriptorReader) retrieve(READER_PROPERTY);
   }
 
@@ -123,11 +123,11 @@ public final class DescriptorSourceFactory {
    *
    * @since 1.0.0
    */
-  public static final DescriptorCollector createDescriptorCollector() {
+  public static DescriptorCollector createDescriptorCollector() {
     return (DescriptorCollector) retrieve(COLLECTOR_PROPERTY);
   }
 
-  private static final <T> Object retrieve(String type) {
+  private static <T> Object retrieve(String type) {
     Object object;
     String clazzName = null;
     try {
@@ -146,10 +146,9 @@ public final class DescriptorSourceFactory {
               + ".");
       }
       object = ClassLoader.getSystemClassLoader().loadClass(clazzName)
-          .newInstance();
+          .getDeclaredConstructor().newInstance();
       log.info("Serving implementation {} for {}.", clazzName, type);
-    } catch (ClassNotFoundException | InstantiationException
-             | IllegalAccessException ex) {
+    } catch (ReflectiveOperationException ex) {
       throw new RuntimeException("Cannot load class "
           + clazzName + "for type " + type, ex);
     }
