@@ -1,4 +1,4 @@
-/* Copyright 2014--2018 The Tor Project
+/* Copyright 2014--2020 The Tor Project
  * See LICENSE for licensing information */
 
 package org.torproject.descriptor.impl;
@@ -30,7 +30,6 @@ public class MicrodescriptorImpl extends DescriptorImpl
     this.checkAtMostOnceKeys(atMostOnceKeys);
     this.checkFirstKey(Key.ONION_KEY);
     this.clearParsedKeys();
-    return;
   }
 
   private void parseDescriptorBytes() throws DescriptorParseException {
@@ -72,6 +71,10 @@ public class MicrodescriptorImpl extends DescriptorImpl
           crypto.append(line).append(NL);
           break;
         case CRYPTO_END:
+          if (null == crypto) {
+            throw new DescriptorParseException(Key.CRYPTO_END + " before "
+                + Key.CRYPTO_BEGIN);
+          }
           crypto.append(line).append(NL);
           String cryptoString = crypto.toString();
           crypto = null;
@@ -170,8 +173,8 @@ public class MicrodescriptorImpl extends DescriptorImpl
         case "accept":
         case "reject":
           String[] ports = parts[2].split(",", -1);
-          for (int i = 0; i < ports.length; i++) {
-            if (ports[i].length() < 1) {
+          for (String port : ports) {
+            if (port.length() < 1) {
               isValid = false;
               break;
             }
