@@ -1,4 +1,4 @@
-/* Copyright 2016--2018 The Tor Project
+/* Copyright 2016--2020 The Tor Project
  * See LICENSE for licensing information */
 
 package org.torproject.descriptor.index;
@@ -10,6 +10,7 @@ import com.fasterxml.jackson.annotation.JsonInclude;
 import com.fasterxml.jackson.annotation.JsonProperty;
 import com.fasterxml.jackson.annotation.JsonPropertyOrder;
 import com.fasterxml.jackson.annotation.PropertyAccessor;
+import com.fasterxml.jackson.databind.DeserializationFeature;
 import com.fasterxml.jackson.databind.ObjectMapper;
 import com.fasterxml.jackson.databind.PropertyNamingStrategy;
 
@@ -48,13 +49,14 @@ public class IndexNode {
 
   /** An empty node, which is not added to JSON output. */
   public static final IndexNode emptyNode = new IndexNode("", "",
-      new TreeSet<FileNode>(), new TreeSet<DirectoryNode>());
+      new TreeSet<>(), new TreeSet<>());
 
   private static ObjectMapper objectMapper = new ObjectMapper()
       .setPropertyNamingStrategy(PropertyNamingStrategy.SNAKE_CASE)
       .setSerializationInclusion(JsonInclude.Include.NON_EMPTY)
       .setVisibility(PropertyAccessor.ALL, JsonAutoDetect.Visibility.NONE)
-      .setVisibility(PropertyAccessor.FIELD, JsonAutoDetect.Visibility.ANY);
+      .setVisibility(PropertyAccessor.FIELD, JsonAutoDetect.Visibility.ANY)
+      .configure(DeserializationFeature.FAIL_ON_UNKNOWN_PROPERTIES, false);
 
   /** The created date-time is exposed in JSON as 'index_created' field. */
   @JsonProperty("index_created")
@@ -137,13 +139,13 @@ public class IndexNode {
       if (null == currentDir) {
         continue;
       }
-      String currentPath = dirs[0] + "/";
+      StringBuilder currentPath = new StringBuilder(dirs[0] + "/");
       for (int k = 1; k < dirs.length; k++) {
         DirectoryNode dn = findPathIn(dirs[k], currentDir.directories);
         if (null == dn) {
           break;
         } else {
-          currentPath += dirs[k] + "/";
+          currentPath.append(dirs[k]).append("/");
           currentDir = dn;
         }
       }
@@ -152,7 +154,7 @@ public class IndexNode {
       }
       for (FileNode file : currentDir.files) {
         if (file.lastModifiedMillis() > 0) { // only add valid files
-          map.put(currentPath + file.path, file);
+          map.put(currentPath.toString() + file.path, file);
         }
       }
     }
