@@ -1,4 +1,4 @@
-/* Copyright 2012--2018 The Tor Project
+/* Copyright 2012--2020 The Tor Project
  * See LICENSE for licensing information */
 
 package org.torproject.descriptor.impl;
@@ -453,6 +453,9 @@ public class ServerDescriptorImplTest {
     private ServerDescriptorImpl buildDescriptor()
         throws DescriptorParseException {
       byte[] descriptorBytes = this.buildDescriptorBytes();
+      if (null == descriptorBytes) {
+        descriptorBytes = new byte[0];
+      }
       return new RelayServerDescriptorImpl(descriptorBytes,
           new int[] { 0, descriptorBytes.length }, null);
     }
@@ -1825,6 +1828,16 @@ public class ServerDescriptorImplTest {
         MASTER_KEY_ED25519_LINE, ROUTER_SIG_ED25519_LINE);
   }
 
+  @Test
+  public void testEd25519IdentityInvalidCrypto()
+      throws DescriptorParseException {
+    this.thrown.expect(DescriptorParseException.class);
+    this.thrown.expectMessage("CRYPTO_END before CRYPTO_BEGIN");
+    DescriptorBuilder.createWithEd25519Lines("identity-ed25519\n"
+        + "-----END ED25519 CERT-----\n-----BEGIN ED25519 CERT-----",
+        MASTER_KEY_ED25519_LINE, ROUTER_SIG_ED25519_LINE);
+  }
+
   @Test
   public void testEd25519MasterKeyMissing()
       throws DescriptorParseException {
