@@ -1,4 +1,4 @@
-/* Copyright 2012--2018 The Tor Project
+/* Copyright 2012--2020 The Tor Project
  * See LICENSE for licensing information */
 
 package org.torproject.descriptor.impl;
@@ -242,6 +242,10 @@ public abstract class ExtraInfoDescriptorImpl extends DescriptorImpl
           cryptoLines.add(line);
           break;
         case CRYPTO_END:
+          if (null == cryptoLines) {
+            throw new DescriptorParseException(Key.CRYPTO_END + " before "
+                + Key.CRYPTO_BEGIN);
+          }
           cryptoLines.add(line);
           StringBuilder sb = new StringBuilder();
           for (String cryptoLine : cryptoLines) {
